@@ -27,6 +27,7 @@ import java.awt.event.WindowAdapter;
 import java.awt.event.WindowEvent;
 import java.awt.event.WindowListener;
 import java.io.File;
+import java.io.IOException;
 import java.net.URL;
 import java.util.Iterator;
 import java.util.LinkedList;
@@ -53,7 +54,7 @@ import org.sump.analyzer.tools.Tool;
  * <p>
  * This class only provides a simple end-user frontend and no functionality to be used by other code.
  * 
- * @version 0.6
+ * @version 0.7
  * @author Michael "Mr. Sump" Poppitz
  */
 public final class MainWindow extends WindowAdapter implements ActionListener, WindowListener, StatusChangeListener {
@@ -120,6 +121,21 @@ public final class MainWindow extends WindowAdapter implements ActionListener, W
 			return ("Sump's Logic Analyzer Files (*.sla)");
 		}
 	}
+
+	/**
+	 * Inner class defining a File Filter for SLP files.
+	 * 
+	 * @author Michael "Mr. Sump" Poppitz
+	 *
+	 */
+	private class SLPFilter extends FileFilter {
+		public boolean accept(File f) {
+			return (f.isDirectory() || f.getName().toLowerCase().endsWith(".slp"));
+		}
+		public String getDescription() {
+			return ("Sump's Logic Analyzer Project Files (*.slp)");
+		}
+	}
 	
 	/**
 	 * Constructs and displays the main window.
@@ -127,7 +143,8 @@ public final class MainWindow extends WindowAdapter implements ActionListener, W
 	 */
 	public MainWindow() {
 		super();
-		
+		project = new Project();
+
 		frame = new JFrame("Logic Analyzer Client");
 		frame.setIconImage((new ImageIcon("org/sump/analyzer/icons/la.png")).getImage());
 		Container contentPane = frame.getContentPane();
@@ -140,25 +157,35 @@ public final class MainWindow extends WindowAdapter implements ActionListener, W
 		JMenu fileMenu = createMenu("File", fileEntries);
 		mb.add(fileMenu);
 
+		// project menu
+		String[] projectEntries = {"Open Project...", "Save Project as...", };
+		JMenu projectMenu = createMenu("Project", projectEntries);
+		mb.add(projectMenu);
+
 		// device menu
 		String[] deviceEntries = {"Capture...", "Repeat Capture"};
 		JMenu deviceMenu = createMenu("Device", deviceEntries);
 		mb.add(deviceMenu);
 		
 		// diagram menu
-		String[] diagramEntries = {"Zoom In", "Zoom Out", "Default Zoom", "", "Diagram Settings..."};
+		String[] diagramEntries = {"Zoom In", "Zoom Out", "Default Zoom", "", "Diagram Settings...", "Labels..."};
 		diagramMenu = createMenu("Diagram", diagramEntries);
 		mb.add(diagramMenu);
 
 		// tools menu
 		String[] toolClasses = { 	// TODO: should be read from properties
-			"org.sump.analyzer.tools.StateAnalysis"
+				"org.sump.analyzer.tools.StateAnalysis",
+				"org.sump.analyzer.tools.SPIProtocolAnalysis"
 		};
 		List loadedTools = new LinkedList();
 		for (int i = 0; i < toolClasses.length; i++) {
 			try {
 				Class tool = Class.forName(toolClasses[i]);
-				loadedTools.add(tool.newInstance());
+				Object o = tool.newInstance();
+				if (o instanceof Tool)
+					loadedTools.add(o);
+				if (o instanceof Configurable)
+					project.addConfigurable((Configurable)o);
 			} catch (Exception e) { e.printStackTrace(); }
 		}
 
@@ -207,19 +234,23 @@ public final class MainWindow extends WindowAdapter implements ActionListener, W
 		contentPane.add(status, BorderLayout.SOUTH);
 		
 		diagram = new Diagram();
+		project.addConfigurable(diagram);
 		diagram.addStatusChangeListener(this);
 		contentPane.add(new JScrollPane(diagram), BorderLayout.CENTER);
 
 		enableDataDependingFunctions(false);
 
 		frame.setSize(1000, 835);
-		frame.setVisible(true);
 		frame.addWindowListener(this);
 
 		fileChooser = new JFileChooser();
 		fileChooser.addChoosableFileFilter((FileFilter) new SLAFilter());
 
+		projectChooser = new JFileChooser();
+		projectChooser.addChoosableFileFilter((FileFilter) new SLPFilter());
+		
 		controller = new DeviceController();
+		project.addConfigurable(controller);
 	}
 	
 	/**
@@ -238,10 +269,8 @@ public final class MainWindow extends WindowAdapter implements ActionListener, W
 			if (label.equals("Open...")) {
 				if (fileChooser.showOpenDialog(frame) == JFileChooser.APPROVE_OPTION) {
 					File file = fileChooser.getSelectedFile();
-					if (file.isFile()) {
-						System.out.println("Opening: " + file.getName() + ".");
-						diagram.setCapturedData(new CapturedData(file));
-					}
+					if (file.isFile())
+						loadData(file);
 				}
 			
 			} else if (label.equals("Save as...")) {
@@ -250,6 +279,20 @@ public final class MainWindow extends WindowAdapter implements ActionListener, W
 					System.out.println("Saving: " + file.getName() + ".");
 					diagram.getCapturedData().writeToFile(file);
 				}
+
+			} else if (label.equals("Open Project...")) {
+				if (projectChooser.showOpenDialog(frame) == JFileChooser.APPROVE_OPTION) {
+					File file = projectChooser.getSelectedFile();
+					if (file.isFile())
+						loadProject(file);
+				}
+				
+			} else if (label.equals("Save Project as...")) {
+				if (projectChooser.showSaveDialog(frame) == JFileChooser.APPROVE_OPTION) {
+					File file = projectChooser.getSelectedFile();
+					System.out.println("Saving Project: " + file.getName() + ".");
+					project.store(file);
+				}
 			
 			} else if (label.equals("Capture...")) {
 				if (controller.showCaptureDialog(frame) == DeviceController.DONE) {
@@ -275,6 +318,9 @@ public final class MainWindow extends WindowAdapter implements ActionListener, W
 
 			} else if (label.equals("Diagram Settings...")) {
 				diagram.showSettingsDialog(frame);
+				
+			} else if (label.equals("Labels...")) {
+				diagram.showLabelsDialog(frame);
 
 			} else if (label.equals("About")) {
 				JOptionPane.showMessageDialog(null,
@@ -316,13 +362,33 @@ public final class MainWindow extends WindowAdapter implements ActionListener, W
 	public void windowClosing(WindowEvent event) {
 		exit();
 	}
+
+	/**
+	 * Load the given file as data.
+	 * @param file file to be loaded as data
+	 * @throws IOException when an IO error occurs
+	 */
+	public void loadData(File file) throws IOException {
+		System.out.println("Opening: " + file.getName());
+		diagram.setCapturedData(new CapturedData(file));
+	}
+	
+	/**
+	 * Load the given file as project.
+	 * @param file file to be loaded as projects
+	 * @throws IOException when an IO error occurs
+	 */
+	public void loadProject(File file) throws IOException {
+		System.out.println("Opening Project: " + file.getName());
+		project.load(file);
+	}
 	
 	/**
 	 * Home of the main thread.
 	 * Which happens to be a lazy one.
 	 */
-	// TODO: Check if it really needs to live as long as the program runs.
 	public void run() {
+		frame.setVisible(true);
 		thread = Thread.currentThread();
 		done = false;
 		while (!done) {
@@ -335,7 +401,7 @@ public final class MainWindow extends WindowAdapter implements ActionListener, W
 	}
 	
 	/**
-	 * Tells the main thread to exit. This will stop stop the VM.
+	 * Tells the main thread to exit. This will stop the VM.
 	 */
 	public void exit() {
 		done = true;
@@ -346,22 +412,14 @@ public final class MainWindow extends WindowAdapter implements ActionListener, W
 		}
 	}
 		
-	/**
-	 * Starts up the logic analyzer client.
-	 * @param args	no arguments are supported
-	 */
-	public static void main(String[] args) {
-		MainWindow w = new MainWindow();
-		w.run();
-		System.exit(0);
-	}
-
 	private JMenu toolMenu;
 	private JMenu diagramMenu;
 	
 	private JFileChooser fileChooser;
+	private JFileChooser projectChooser;
 	private DeviceController controller;
 	private Diagram diagram;
+	private Project project;
 	private JLabel status;
 	private Tool[] tools;
 	
